# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .dimensional_price_groups import (
    DimensionalPriceGroups,
    AsyncDimensionalPriceGroups,
    DimensionalPriceGroupsWithRawResponse,
    AsyncDimensionalPriceGroupsWithRawResponse,
    DimensionalPriceGroupsWithStreamingResponse,
    AsyncDimensionalPriceGroupsWithStreamingResponse,
)
from .external_dimensional_price_group_id import (
    ExternalDimensionalPriceGroupID,
    AsyncExternalDimensionalPriceGroupID,
    ExternalDimensionalPriceGroupIDWithRawResponse,
    AsyncExternalDimensionalPriceGroupIDWithRawResponse,
    ExternalDimensionalPriceGroupIDWithStreamingResponse,
    AsyncExternalDimensionalPriceGroupIDWithStreamingResponse,
)

__all__ = [
    "ExternalDimensionalPriceGroupID",
    "AsyncExternalDimensionalPriceGroupID",
    "ExternalDimensionalPriceGroupIDWithRawResponse",
    "AsyncExternalDimensionalPriceGroupIDWithRawResponse",
    "ExternalDimensionalPriceGroupIDWithStreamingResponse",
    "AsyncExternalDimensionalPriceGroupIDWithStreamingResponse",
    "DimensionalPriceGroups",
    "AsyncDimensionalPriceGroups",
    "DimensionalPriceGroupsWithRawResponse",
    "AsyncDimensionalPriceGroupsWithRawResponse",
    "DimensionalPriceGroupsWithStreamingResponse",
    "AsyncDimensionalPriceGroupsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .threshold_param import ThresholdParam

__all__ = ["AlertUpdateParams"]


class AlertUpdateParams(TypedDict, total=False):
    thresholds: Required[Iterable[ThresholdParam]]
    """The thresholds that define the values at which the alert will be triggered."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias

from .trial_discount import TrialDiscount
from .usage_discount import UsageDiscount
from .amount_discount import AmountDiscount
from .percentage_discount import PercentageDiscount

__all__ = ["Discount"]

Discount: TypeAlias = Union[PercentageDiscount, TrialDiscount, UsageDiscount, AmountDiscount]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .shared.aggregated_cost import AggregatedCost

__all__ = ["SubscriptionFetchCostsResponse"]


class SubscriptionFetchCostsResponse(BaseModel):
    data: List[AggregatedCost]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .conversion_rate_tier import ConversionRateTier

__all__ = ["ConversionRateTieredConfig"]


class ConversionRateTieredConfig(BaseModel):
    tiers: List[ConversionRateTier]
    """Tiers for rating based on total usage quantities into the specified tier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["NewBillingCycleConfiguration"]


class NewBillingCycleConfiguration(TypedDict, total=False):
    duration: Required[int]
    """The duration of the billing period."""

    duration_unit: Required[Literal["day", "month"]]
    """The unit of billing period duration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["BillingCycleAnchorConfiguration"]


class BillingCycleAnchorConfiguration(TypedDict, total=False):
    day: Required[int]
    """The day of the month on which the billing cycle is anchored.

    If the maximum number of days in a month is greater than this value, the last
    day of the month is the billing cycle day (e.g. billing_cycle_day=31 for April
    means the billing period begins on the 30th.
    """

    month: Optional[int]
    """The month on which the billing cycle is anchored (e.g.

    a quarterly price anchored in February would have cycles starting February, May,
    August, and November).
    """

    year: Optional[int]
    """The year on which the billing cycle is anchored (e.g.

    a 2 year billing cycle anchored on 2021 would have cycles starting on 2021,
    2023, 2025, etc.).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from .trial_discount import TrialDiscount
from .amount_discount import AmountDiscount
from .percentage_discount import PercentageDiscount

__all__ = ["InvoiceLevelDiscount"]

InvoiceLevelDiscount: TypeAlias = Annotated[
    Union[PercentageDiscount, AmountDiscount, TrialDiscount], PropertyInfo(discriminator="discount_type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .sub_line_item_grouping import SubLineItemGrouping

__all__ = ["TierSubLineItem", "TierConfig"]


class TierConfig(BaseModel):
    first_unit: float

    last_unit: Optional[float] = None

    unit_amount: str


class TierSubLineItem(BaseModel):
    amount: str
    """The total amount for this sub line item."""

    grouping: Optional[SubLineItemGrouping] = None

    name: str

    quantity: float

    tier_config: TierConfig

    type: Literal["tier"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .tier import Tier

__all__ = ["TieredConfig"]


class TieredConfig(TypedDict, total=False):
    tiers: Required[Iterable[Tier]]
    """Tiers for rating based on total usage quantities into the specified tier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["UnitConfig"]


class UnitConfig(TypedDict, total=False):
    unit_amount: Required[str]
    """Rate per unit of usage"""

    scaling_factor: Optional[float]
    """Multiplier to scale rated quantity by"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TransformPriceFilter"]


class TransformPriceFilter(BaseModel):
    field: Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]
    """The property of the price to filter on."""

    operator: Literal["includes", "excludes"]
    """Should prices that match the filter be included or excluded."""

    values: List[str]
    """The IDs or values that match this filter."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .conversion_rate_unit_config import ConversionRateUnitConfig

__all__ = ["UnitConversionRateConfig"]


class UnitConversionRateConfig(TypedDict, total=False):
    conversion_rate_type: Required[Literal["unit"]]

    unit_config: Required[ConversionRateUnitConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from ..dimensional_price_group import DimensionalPriceGroup
from ..shared.pagination_metadata import PaginationMetadata

__all__ = ["DimensionalPriceGroups"]


class DimensionalPriceGroups(BaseModel):
    data: List[DimensionalPriceGroup]

    pagination_metadata: PaginationMetadata



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["NewDimensionalPriceConfiguration"]


class NewDimensionalPriceConfiguration(BaseModel):
    dimension_values: List[str]
    """
    The list of dimension values matching (in order) the dimensions of the price
    group
    """

    dimensional_price_group_id: Optional[str] = None
    """The id of the dimensional price group to include this price in"""

    external_dimensional_price_group_id: Optional[str] = None
    """The external id of the dimensional price group to include this price in"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CustomerMinified"]


class CustomerMinified(BaseModel):
    id: str

    external_customer_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal

import httpx

from ... import _legacy_response
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ...pagination import SyncPage, AsyncPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.customers import balance_transaction_list_params, balance_transaction_create_params
from ...types.customers.balance_transaction_list_response import BalanceTransactionListResponse
from ...types.customers.balance_transaction_create_response import BalanceTransactionCreateResponse

__all__ = ["BalanceTransactions", "AsyncBalanceTransactions"]


class BalanceTransactions(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BalanceTransactionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/orbcorp/orb-python#accessing-raw-response-data-eg-headers
        """
        return BalanceTransactionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BalanceTransactionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/orbcorp/orb-python#with_streaming_response
        """
        return BalanceTransactionsWithStreamingResponse(self)

    def create(
        self,
        customer_id: str,
        *,
        amount: str,
        type: Literal["increment", "decrement"],
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> BalanceTransactionCreateResponse:
        """
        Creates an immutable balance transaction that updates the customer's balance and
        returns back the newly created transaction.

        Args:
          description: An optional description that can be specified around this entry.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._post(
            f"/customers/{customer_id}/balance_transactions",
            body=maybe_transform(
                {
                    "amount": amount,
                    "type": type,
                    "description": description,
                },
                balance_transaction_create_params.BalanceTransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=BalanceTransactionCreateResponse,
        )

    def list(
        self,
        customer_id: str,
        *,
        cursor: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        operation_time_gt: Union[str, datetime, None] | Omit = omit,
        operation_time_gte: Union[str, datetime, None] | Omit = omit,
        operation_time_lt: Union[str, datetime, None] | Omit = omit,
        operation_time_lte: Union[str, datetime, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[BalanceTransactionListResponse]:
        """
        ## The customer balance

        The customer balance is an amount in the customer's currency, which Orb
        automatically applies to subsequent invoices. This balance can be adjusted
        manually via Orb's webapp on the customer details page. You can use this balance
        to provide a fixed mid-period credit to the customer. Commonly, this is done due
        to system downtime/SLA violation, or an adhoc adjustment discussed with the
        customer.

        If the balance is a positive value at the time of invoicing, it represents that
        the customer has credit that should be used to offset the amount due on the next
        issued invoice. In this case, Orb will automatically reduce the next invoice by
        the balance amount, and roll over any remaining balance if the invoice is fully
        discounted.

        If the balance is a negative value at the time of invoicing, Orb will increase
        the invoice's amount due with a positive adjustment, and reset the balance to 0.

        This endpoint retrieves all customer balance transactions in reverse
        chronological order for a single customer, providing a complete audit trail of
        all adjustments and invoice applications.

        Args:
          cursor: Cursor for pagination. This can be populated by the `next_cursor` value returned
              from the initial request.

          limit: The number of items to fetch. Defaults to 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/customers/{customer_id}/balance_transactions",
            page=SyncPage[BalanceTransactionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "operation_time_gt": operation_time_gt,
                        "operation_time_gte": operation_time_gte,
                        "operation_time_lt": operation_time_lt,
                        "operation_time_lte": operation_time_lte,
                    },
                    balance_transaction_list_params.BalanceTransactionListParams,
                ),
            ),
            model=BalanceTransactionListResponse,
        )


class AsyncBalanceTransactions(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBalanceTransactionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/orbcorp/orb-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBalanceTransactionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBalanceTransactionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/orbcorp/orb-python#with_streaming_response
        """
        return AsyncBalanceTransactionsWithStreamingResponse(self)

    async def create(
        self,
        customer_id: str,
        *,
        amount: str,
        type: Literal["increment", "decrement"],
        description: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> BalanceTransactionCreateResponse:
        """
        Creates an immutable balance transaction that updates the customer's balance and
        returns back the newly created transaction.

        Args:
          description: An optional description that can be specified around this entry.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return await self._post(
            f"/customers/{customer_id}/balance_transactions",
            body=await async_maybe_transform(
                {
                    "amount": amount,
                    "type": type,
                    "description": description,
                },
                balance_transaction_create_params.BalanceTransactionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=BalanceTransactionCreateResponse,
        )

    def list(
        self,
        customer_id: str,
        *,
        cursor: Optional[str] | Omit = omit,
        limit: int | Omit = omit,
        operation_time_gt: Union[str, datetime, None] | Omit = omit,
        operation_time_gte: Union[str, datetime, None] | Omit = omit,
        operation_time_lt: Union[str, datetime, None] | Omit = omit,
        operation_time_lte: Union[str, datetime, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BalanceTransactionListResponse, AsyncPage[BalanceTransactionListResponse]]:
        """
        ## The customer balance

        The customer balance is an amount in the customer's currency, which Orb
        automatically applies to subsequent invoices. This balance can be adjusted
        manually via Orb's webapp on the customer details page. You can use this balance
        to provide a fixed mid-period credit to the customer. Commonly, this is done due
        to system downtime/SLA violation, or an adhoc adjustment discussed with the
        customer.

        If the balance is a positive value at the time of invoicing, it represents that
        the customer has credit that should be used to offset the amount due on the next
        issued invoice. In this case, Orb will automatically reduce the next invoice by
        the balance amount, and roll over any remaining balance if the invoice is fully
        discounted.

        If the balance is a negative value at the time of invoicing, Orb will increase
        the invoice's amount due with a positive adjustment, and reset the balance to 0.

        This endpoint retrieves all customer balance transactions in reverse
        chronological order for a single customer, providing a complete audit trail of
        all adjustments and invoice applications.

        Args:
          cursor: Cursor for pagination. This can be populated by the `next_cursor` value returned
              from the initial request.

          limit: The number of items to fetch. Defaults to 20.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not customer_id:
            raise ValueError(f"Expected a non-empty value for `customer_id` but received {customer_id!r}")
        return self._get_api_list(
            f"/customers/{customer_id}/balance_transactions",
            page=AsyncPage[BalanceTransactionListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "operation_time_gt": operation_time_gt,
                        "operation_time_gte": operation_time_gte,
                        "operation_time_lt": operation_time_lt,
                        "operation_time_lte": operation_time_lte,
                    },
                    balance_transaction_list_params.BalanceTransactionListParams,
                ),
            ),
            model=BalanceTransactionListResponse,
        )


class BalanceTransactionsWithRawResponse:
    def __init__(self, balance_transactions: BalanceTransactions) -> None:
        self._balance_transactions = balance_transactions

        self.create = _legacy_response.to_raw_response_wrapper(
            balance_transactions.create,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            balance_transactions.list,
        )


class AsyncBalanceTransactionsWithRawResponse:
    def __init__(self, balance_transactions: AsyncBalanceTransactions) -> None:
        self._balance_transactions = balance_transactions

        self.create = _legacy_response.async_to_raw_response_wrapper(
            balance_transactions.create,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            balance_transactions.list,
        )


class BalanceTransactionsWithStreamingResponse:
    def __init__(self, balance_transactions: BalanceTransactions) -> None:
        self._balance_transactions = balance_transactions

        self.create = to_streamed_response_wrapper(
            balance_transactions.create,
        )
        self.list = to_streamed_response_wrapper(
            balance_transactions.list,
        )


class AsyncBalanceTransactionsWithStreamingResponse:
    def __init__(self, balance_transactions: AsyncBalanceTransactions) -> None:
        self._balance_transactions = balance_transactions

        self.create = async_to_streamed_response_wrapper(
            balance_transactions.create,
        )
        self.list = async_to_streamed_response_wrapper(
            balance_transactions.list,
        )
